local ADDON, data = ...

local UnitAura = UnitAura

local lstFlaskEffects = {
	-- Pandaria flasks
    --[105689] = true,
	--[105691] = true,
	--[105693] = true,
	--[105694] = true,
	--[105696] = true,	
    
    -- Draenor
    [156073] = true,    -- agi
    [156070] = true,    -- int
    [156071] = true,    -- str
    [156077] = true,    -- stam
    [156064] = true,    -- greater agi
    [156079] = true,    -- greater int
    [156080] = true,    -- greater str
    [156084] = true,    -- greater stam

	--[79634] = true, -- Golemblood potion, for testing
    --[53758] = true, -- Stoneblood flask, for testing
    --[124974] = true, -- Nature's Vigil, for testing
}

local function IsFlaskEffect (idSpell)
    --return auraName and auraName:match ('Flask') --or auraName:match('Elixir')
	return lstFlaskEffects[idSpell]==true
end

lstFlaskItems = {
	-- MoP
    --[76084] = true,
	--[76085] = true,
	--[76086] = true,
	--[76087] = true,
	--[76088] = true,
	
    -- Draenor
    [109145] = true,    -- agi
    [109147] = true,    -- int
    [109148] = true,    -- str
    [109152] = true,    -- stam
    [109153] = true,    -- greater agi
    [109155] = true,    -- greater int
    [109156] = true,    -- greater str
    [109160] = true,    -- greater stam

	--[58146] = true, -- Golemblood potion, for testing
    --[46379] = true, -- Stoneblood flask, for testing
}

local function IsFlaskItem (idItem)
	return lstFlaskItems[tonumber(idItem)]==true
end

------------------------------------------------------------------
local function dprint (...)
    if DEBUG and DEBUG[ADDON] then
        print (...)
    end
end
------------------------------------------------------------------

local function ForEachItemInBag (func)
    -- func should be of the form foo(nBag, nSlot, itemLink), and return false if it wants to stop iterating
    local bag, slot, count, link
    for bag=0, NUM_BAG_SLOTS do
        for slot = 1, GetContainerNumSlots(bag) do
            local _,count,_,_,_,_,link = GetContainerItemInfo(bag, slot)
            if link then 
                func (bag, slot, link)
            end
        end
    end
end

------------------------------------------------------------------

local function CreateFlaskButton ()
    local size = 36
    local f = CreateFrame ("Frame", "frmFlaskaholic", UIParent)
    f:SetSize (size,size)
    --f:SetPoint ("BOTTOMLEFT", UIParent, "BOTTOMLEFT", 8, 38)
    f:SetPoint ("CENTER", UIParent, "CENTER")
    f.name = ''
    
    -- Button
    f.btn = CreateFrame ("Button", '', f, "SecureActionButtonTemplate")
    f.btn:SetPoint ("BOTTOM", f, "BOTTOM")
    f.btn:SetSize (size, size)
    f.btn:SetNormalTexture ("Interface\\ICONS\\INV_Potion_107")
    --f.btn:SetHighlightTexture ("Interface\\Buttons\\BLUEGRAD64", "ADD")
    f.btn:SetHighlightTexture ("Interface\\Addons\\Flaskaholic\\highlight", "ADD")
    f.btn:RegisterForClicks ("AnyUp")
    f.btn:SetAttribute ("type", "item")
    
    -- Create the text    
    f.txt = CreateFrame ('MessageFrame', nil, f)
    f.txt:SetSize (size, size/2.3)
    f.txt:SetPoint ('BOTTOMRIGHT', f, 'BOTTOMRIGHT')
    f.txt:SetInsertMode ('BOTTOM')
    f.txt:SetFading (false)
    f.txt:SetTextColor (1.0,1.0,1.0,1.0)
    f.txt:SetFont ('Fonts\\FRIZQT__.TTF', size/3, "THICKOUTLINE")
    f.txt:SetJustifyV('BOTTOM')
    f.txt:SetJustifyH('RIGHT')
    f.txt:SetFrameLevel(f:GetFrameLevel()+1)
    f.txt:SetShadowColor (0.0, 0.0, 0.0, 1.0)
    f.txt:SetShadowOffset (-1, -1)
    
    -- Moveme
    f.moveme = CreateFrame ('MessageFrame', nil, f)
    f.moveme:SetSize (size, 20)
    f.moveme:SetPoint ('TOP', f, 'BOTTOM', 0, 0)
    f.moveme:SetFrameStrata ('TOOLTIP')
    f.moveme:SetInsertMode ('TOP')
    f.moveme:SetFading (false)
    f.moveme:SetTextColor (1.0,1.0,1.0,1.0)
    f.moveme:SetFont ('Fonts\\FRIZQT__.TTF', 8, "")
    f.moveme:SetJustifyV('TOP')
    f.moveme:AddMessage ('Move me')
    f.moveme:SetBackdrop ({
        bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",  
        edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Border",
        edgeSize = 8,
    })
    f.moveme:Hide()
    
    -- Toggle
    f.toggle = CreateFrame ('Frame', nil, f)
    f.toggle:SetSize (size, 22)
    f.toggle:SetPoint ('BOTTOM', f, 'TOP', 0, 0)
    f.toggle:SetFrameStrata ('TOOLTIP')
    f.toggle:SetBackdrop ({
        bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",  
        edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Border",
        edgeSize = 8,
    })
    f.toggle:Hide()

    f.disableLight = false
    f.btnToggle = CreateFrame ("Button", nil, f.toggle, "UIPanelButtonTemplate")
    f.btnToggle:SetAllPoints (f.toggle)
    f.btnToggle:SetText ('')
    f.btnToggle:SetScript ("OnClick", function (...)
        f.disableLight = not f.disableLight
    end)
    
    -- Picker
    f.picker = CreateFrame ('Frame', nil, f)
    f.picker:SetSize (14, 26)
    f.picker:SetPoint ('LEFT', f, 'RIGHT')
    f.picker:SetFrameStrata ('TOOLTIP')
    f.picker:Hide()
    
    f.btnPicker = CreateFrame ("Button", nil, f.picker)
    f.btnPicker:SetAllPoints (f.picker)
    f.btnPicker:SetNormalTexture ("Interface\\Addons\\Flaskaholic\\arrowright")
    --f.btnPicker:SetPushedTexture ("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Down")
    f.btnPicker:SetHighlightTexture ("Interface\\Buttons\\UI-Common-MouseHilight", "ADD")
        
    -- Actual picking action
    f.tblPickerFrames = {[0]=f}
    
	function f:AddPickFrame (nBag, nSlot, name, texture)
        local size = 36
        local f = CreateFrame ("Frame", nil, self)
        f:SetSize (size,size)
        f:SetFrameStrata ('TOOLTIP')
        f.nBag = nBag
        f.nSlot = nSlot
        f.name = name
        
        f.tex = f:CreateTexture ('Texture')
        f.tex:SetAllPoints (f)
        f.tex:SetTexture ("Interface\\BUTTONS\\UI-Slot-Background")
        f.tex:SetTexCoord (0.0, 0.64, 0.0, 0.64)
    
        f.btn = CreateFrame ("Button", nil, f)
        f.btn:SetPoint ("TOPLEFT", f, "TOPLEFT", 2, -2)
        f.btn:SetSize (size-4,size-4)
        f.btn:SetNormalTexture (texture)    
        f.btn:SetHighlightTexture ("Interface\\Buttons\\UI-Common-MouseHilight", "ADD")
        
        f.btn:SetScript ("OnEnter", function (self,...)
            GameTooltip:SetOwner (self:GetParent(), "ANCHOR_TOP")
            GameTooltip:SetBagItem (self:GetParent().nBag, self:GetParent().nSlot)
            GameTooltip:Show ()
        end)
        f.btn:SetScript ("OnLeave", function (...)
            GameTooltip:Hide ()
        end)
        f.btn:SetScript ("OnClick", function (self, ...) 
            frmFlaskaholic:LoadItem (self:GetParent().name)
            frmFlaskaholic:StopPicking ()
        end)
        
        f:SetScript ("OnEnter", f.btn:GetScript("OnEnter"))
        f:SetScript ("OnLeave", f.btn:GetScript("OnLeave"))
        
        f:SetPoint ("LEFT", self.tblPickerFrames[#self.tblPickerFrames], "RIGHT")
        table.insert (self.tblPickerFrames, f)
    end
    
	function f:StartPicking ()
        ForEachItemInBag (function (nBag, nSlot, itemLink)
            local name,link,_,_,_,_,_,_,_,texture = GetItemInfo(itemLink)
			local _, _, _, _, idItem = string.find (link,"|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*)")
			if IsFlaskItem (idItem) then
				local add=true
                for i=1,#self.tblPickerFrames do
                    if self.tblPickerFrames[i].name==name then
                        add = false
                    end
                end
                
                if add then
                    self:AddPickFrame (nBag, nSlot, name, texture)
                end
            end
        end)
        
        self.picker:SetPoint ('LEFT', self.tblPickerFrames[#self.tblPickerFrames], 'RIGHT')
        self.btnPicker:SetNormalTexture ("Interface\\Addons\\Flaskaholic\\arrowleft")
        self.isPicking = true    
    end
    
	function f:StopPicking ()
        local i=1
        while self.tblPickerFrames[i] do
            self.tblPickerFrames[i]:Hide()
            self.tblPickerFrames[i] = nil
            i = i + 1
        end
        
        self.picker:SetPoint ('LEFT', self.tblPickerFrames[#self.tblPickerFrames], 'RIGHT')
        self.picker:Hide()
        self.btnPicker:SetNormalTexture ("Interface\\Addons\\Flaskaholic\\arrowright")
        self.isPicking = false
        self:Light ()
    end
    
	f.btnPicker:SetScript ("OnClick", function (...)
        if not f.isPicking then
            f:StartPicking()
        else
            f:StopPicking()
        end
    end)

    -- Cooldown
    f.cd = CreateFrame ('Cooldown', nil, frm)
    f.cd:SetAllPoints (f)
    f.cd:SetReverse (true)
    
    -- Enable moving
    f:SetMovable (true)
    f.moveme:RegisterForDrag ("LeftButton")
    f:SetUserPlaced (true)
    f:SetClampedToScreen (true)
    f.moveme:EnableMouse (true)
    f.moveme:SetScript("OnMouseDown", function(self, button)
        f:StartMoving ()
    end)
    f.moveme:SetScript("OnMouseUp", function(self, button)
        f:StopMovingOrSizing ()
    end)
    
    -- Item count
    function f:Count ()
        local count = GetItemCount (self.name, false, true)
        self.txt:Clear ()
        if self.name~='' then
            self.txt:AddMessage (count)
        end
    end
    
    -- Add a tooltip... and other OnEnter/ OnLeave stuff
    function f:ShowTooltip(motion, ...)
        GameTooltip:SetOwner (f, "ANCHOR_RIGHT")
        if f.name=='' then
            -- No current flask
            GameTooltip:AddLine ('Flaskaholic', 1.0, 1.0, 0.0)
            GameTooltip:AddLine ('Once your flask effect runs out, this frame will light up. Press it to drink a new similar flask!', 1.0, 1.0, 1.0, true)
        else
            local count = GetItemCount (f.name, false, true)
            if count==0 then
                GameTooltip:AddLine ("You're all out of "..f.name.." :(", 1.0, 0.2, 0.2)
            else
                GameTooltip:AddLine ("Click to drink a new "..f.name.."!", 1.0, 1.0, 1.0)
            end
        end
        GameTooltip:Show ()
        f.moveme:Show()
        f.toggle:Show()
        f.picker:Show()
        --f:Dark()
    end
    
	function f:HideTooltip (motion, ...)
        GameTooltip:Hide()
        f.moveme:Hide()
        f.toggle:Hide()
        if not f.isPicking then
            f.picker:Hide()
        end
    end
    
	-- Now, bind it
    f:SetScript             ("OnEnter",     f.ShowTooltip)
    f:SetScript             ("OnLeave",     f.HideTooltip)
    f.btn:SetScript         ("OnEnter",     f.ShowTooltip)
    f.btn:SetScript         ("OnLeave",     f.HideTooltip)
    f.moveme:SetScript      ("OnEnter", 	f.moveme.Show)
    f.moveme:SetScript      ("OnLeave",     f.moveme.Hide)
    
    local function OnEnterToggle ()
        f.toggle:Show()
        GameTooltip:SetOwner (f.toggle, "ANCHOR_RIGHT")
        GameTooltip:AddLine ('Toggle', 1.0, 1.0, 1.0)
        GameTooltip:Show ()
    end
    
	local function OnLeaveToggle ()
        f.toggle:Hide()
        GameTooltip:Hide ()
    end
    
	f.toggle:SetScript      ("OnEnter",  	OnEnterToggle)
    f.btnToggle:SetScript   ("OnEnter",     OnEnterToggle)
    f.toggle:SetScript      ("OnLeave",     OnLeaveToggle)
    f.btnToggle:SetScript   ("OnLeave",     OnLeaveToggle)
    
    local function OnEnterPicker ()
        if not f.isPicking then
            f.picker:Show()
            GameTooltip:SetOwner (f.picker, "ANCHOR_RIGHT")
            GameTooltip:AddLine ('Pick a flask', 1.0, 1.0, 1.0)
            GameTooltip:Show ()
        end
    end
    
	local function OnLeavePicker ()
        if not f.isPicking then
            f.picker:Hide()
        end
        GameTooltip:Hide ()
    end
    
	f.picker:SetScript      ("OnEnter",		OnEnterPicker)
    f.btnPicker:SetScript   ("OnEnter",     OnEnterPicker)
    f.picker:SetScript      ("OnLeave",     OnLeavePicker)
    f.btnPicker:SetScript   ("OnLeave",     OnLeavePicker)
    
	-- Timer
	f.animGroup = f:CreateAnimationGroup()
    f.anim = f.animGroup:CreateAnimation()
	
	function f:Timer (seconds, func)
		f.anim:SetDuration (seconds)
		f.animGroup:SetScript ("OnFinished", function (self, requested)
			if not requested then
				func ()
			end
		end)
        f.animGroup:Play ()
	end
	
    -- Events
    function f:Dark ()
        f.btn:UnlockHighlight ()
        f:Count ()
        g_highlit = false
    end
    function f:Light ()
        f.btn:LockHighlight ()
        f:Count ()
        g_highlit = true
        --f:RegisterUnitEvent ('UNIT_SPELLCAST_SUCCEEDED', 'player')
    end
	
    function f:LoadItem (name, texture)
        dprint ('loaditem',name,texture)
        if name then
            f.name = name
            f:Count()
            
            texture = texture or GetItemIcon(name)
            
            if texture then
                f.btn:SetNormalTexture (texture)
            end
            f.btn:SetAttribute ("item", name)
            g_lastFlask = {name=name, texture=GetItemIcon(name)}
        end
    end
    
	
	f:RegisterUnitEvent ('UNIT_SPELLCAST_SUCCEEDED', 'player')
    f:RegisterUnitEvent ('UNIT_AURA', 'player')
    f:RegisterEvent ('BAG_UPDATE_DELAYED')
	f:RegisterEvent ('PLAYER_LOGIN')
	f:SetScript("OnEvent", function(self, event, ...) 
        return f[event] and f[event](f, ...) 
    end)
    
    do
        local expires, name, idSpell = 0,nil,nil
        local i
        function f:UNIT_AURA ()
            i = 1
            repeat
                name,_,_,_,_,_, expires,_,_,_, idSpell = UnitAura ('player', i)
                if IsFlaskEffect (idSpell) then   
                    dprint ('found flask!')
                    f:LoadItem (name)
                    break
                end
                i = i+1
            until name==nil
        end
    
        local t, timeLeft = 0,0
        f:SetScript("OnUpdate", function (self, elapsed)
            t = t + elapsed
            if t > 0.5 then
                t = 0
                
                timeLeft = (expires or 0) - GetTime ()
                if timeLeft > 5*60 then
                    f:Dark ()
                    f.disableLight = false
                    dprint('dark!')
                elseif f.disableLight then
                    f:Dark ()
                elseif timeLeft > 1*60 or timeLeft <= 0 then
                    f:Light ()
                    dprint('light!')
                    --f:Timer (expires-GetTime()-60*5, function() f:Light() end)
                else
                    if g_highlit then
                        f:Dark ()
                    else
                        f:Light ()
                    end
                    dprint('blink!')
                end
                
            end
        end)
    end
    
    function f:PLAYER_LOGIN ()
        if g_highlit==true then
            self:Light()
        else
            self:Dark()
        end
        if not g_lastFlask then
            g_lastFlask = {}
        else
            f:LoadItem (g_lastFlask.name, g_lastFlask.texture)
        end
    end
    
    f.init = false
    function f:BAG_UPDATE_DELAYED (idBag, icon)
        if not f.init then
            f:LoadItem (g_lastFlask.name, g_lastFlask.texture)
        end
        self:Count ()
    end
    
	f:Show ()
    g_highlit = false

    return f
end

CreateFlaskButton ()

------------------------------------------------------------------
